MODULE MODE_INIT_PHYEX
IMPLICIT NONE
CONTAINS
SUBROUTINE FILL_DIMPHYEX(KLON, KLEV, D)
USE MODD_DIMPHYEX,   ONLY: DIMPHYEX_t
USE MODD_PARAMETERS, ONLY: JPVEXT_TURB, JPVEXT
USE MODE_MSG, ONLY: PRINT_MSG, NVERB_FATAL
IMPLICIT NONE
INTEGER, INTENT(IN) :: KLON
INTEGER, INTENT(IN) :: KLEV
TYPE(DIMPHYEX_t), INTENT(OUT) :: D

IF(JPVEXT /= 1 .OR. JPVEXT_TURB /= 1) THEN
  !All arrays are presently declared with klev+2
  CALL PRINT_MSG(NVERB_FATAL, 'GEN', 'FILL_DIMPHYEX', 'For now, JPVEXT and JPVEXT_TURB must be equal to 1')
ENDIF
D%NIT  = KLON
D%NIB  = 1
D%NIE  = KLON
D%NJT  = 1
D%NJB  = 1
D%NJE  = 1
D%NIJT = D%NIT * D%NJT
D%NIJB = 1
D%NIJE = 1
D%NKL  = 1
D%NKT  = KLEV+2
D%NKA  = 1
D%NKU  = KLEV+2
D%NKB  = 1+JPVEXT_TURB
D%NKE  = D%NKT - JPVEXT_TURB
D%NKTB = 1+JPVEXT_TURB
D%NKTE = D%NKT - JPVEXT_TURB
D%NIBC = 1
D%NJBC = 1
D%NIEC = D%NIE
D%NJEC = D%NJT
END SUBROUTINE FILL_DIMPHYEX
!
SUBROUTINE INIT_PHYEX(PTSTEP, PDZMIN, PHYEX)
!
USE MODD_PHYEX,      ONLY: PHYEX_t
USE MODI_INI_PHYEX,  ONLY: INI_PHYEX
USE print_control_mod, ONLY : lunout
!
IMPLICIT NONE
!
REAL, INTENT(IN)              :: PTSTEP
REAL, INTENT(IN)              :: PDZMIN
TYPE(PHYEX_t), INTENT(OUT) :: PHYEX
!
CHARACTER(LEN=6) :: CPROGRAM
CHARACTER(LEN=4) :: CMICRO
CHARACTER(LEN=4) :: CSCONV
CHARACTER(LEN=4) :: CTURB
INTEGER :: K
LOGICAL :: LREADNAM, LOPENED
INTEGER :: IUNITNML, ILUN
!
!General configuration, cannot be modified by namelist
CPROGRAM='LMDZ'
CMICRO='ICE3'
CSCONV='EDKF'
CTURB='TKEL'
!
! Initialize PHYEX
!If the namelist file exists, we use it
INQUIRE(FILE='phyex.nam', EXIST=LREADNAM)
IF(LREADNAM) THEN
  IUNITNML=-1
  DO ILUN=1,100
    INQUIRE(UNIT=ILUN, OPENED=LOPENED)
    IF (.NOT. LOPENED) THEN
      IUNITNML=ILUN
      EXIT
    END IF
  END DO
  OPEN(ACTION='read', FILE='phyex.nam', UNIT=IUNITNML)
ENDIF
CALL INI_PHYEX(HPROGRAM=CPROGRAM, KUNITNML=IUNITNML, LDNEEDNAM=.FALSE., &
              &KLUOUT=lunout, KFROM=0, KTO=1, &
              &PTSTEP=PTSTEP, PDZMIN=PDZMIN, &
              &CMICRO=CMICRO, CSCONV=CSCONV, CTURB=CTURB, &
              &LDDEFAULTVAL=.TRUE., LDREADNAM=LREADNAM, LDCHECK=.TRUE., &
              &KPRINT=2, LDINIT=.TRUE., &
              &PHYEX_OUT=PHYEX)
!
PHYEX%MISC%CPROGRAM=CPROGRAM
PHYEX%MISC%ZTFILE%LOPENED=.FALSE.
PHYEX%MISC%TLES%LLES=.FALSE.
PHYEX%MISC%CMICRO=CMICRO
PHYEX%MISC%CSCONV=CSCONV
PHYEX%MISC%CTURB=CTURB
!
! Budgets
DO K=1, PHYEX%MISC%NBUDGET
  PHYEX%MISC%YLBUDGET(K)%NBUDGET=K
ENDDO
PHYEX%MISC%TBUCONF%LBU_ENABLE=.FALSE.
PHYEX%MISC%TBUCONF%LBUDGET_U=.FALSE.
PHYEX%MISC%TBUCONF%LBUDGET_V=.FALSE.
PHYEX%MISC%TBUCONF%LBUDGET_W=.FALSE.
PHYEX%MISC%TBUCONF%LBUDGET_TH=.FALSE.
PHYEX%MISC%TBUCONF%LBUDGET_TKE=.FALSE.
PHYEX%MISC%TBUCONF%LBUDGET_RV=.FALSE.
PHYEX%MISC%TBUCONF%LBUDGET_RC=.FALSE.
PHYEX%MISC%TBUCONF%LBUDGET_RR=.FALSE.
PHYEX%MISC%TBUCONF%LBUDGET_RI=.FALSE.
PHYEX%MISC%TBUCONF%LBUDGET_RS=.FALSE.
PHYEX%MISC%TBUCONF%LBUDGET_RG=.FALSE.
PHYEX%MISC%TBUCONF%LBUDGET_RH=.FALSE.
PHYEX%MISC%TBUCONF%LBUDGET_SV=.FALSE.
!
END SUBROUTINE INIT_PHYEX
END MODULE MODE_INIT_PHYEX
